@@ -7,7 +7,7 @@ from pdm.cli import actions
 from pdm.cli.commands.base import BaseCommand
 from pdm.cli.hooks import HookManager
 from pdm.cli.options import skip_option
-from pdm.models.backends import _BACKENDS, BuildBackend, get_backend
+from pdm.models.backends import _BACKENDS, DEFAULT_BACKEND, BuildBackend, get_backend
 from pdm.models.python import PythonInfo
 from pdm.project import Project
 from pdm.utils import get_user_email_from_git, get_venv_like_prefix
@@ -38,13 +38,14 @@ class Command(BaseCommand):
         )
         parser.add_argument("--python", help="Specify the Python version/path to use")
         parser.add_argument("--backend", choices=list(_BACKENDS), help="Specify the build backend")
+        parser.add_argument("--lib", action="store_true", help="Create a library project")
         parser.set_defaults(search_parent=False)
 
     def handle(self, project: Project, options: argparse.Namespace) -> None:
         from pdm.cli.commands.venv.utils import get_venv_python
 
         hooks = HookManager(project, options.skip)
-        if project.pyproject.exists:
+        if project.pyproject.exists():
             project.core.ui.echo("pyproject.toml already exists, update it now.", style="primary")
         else:
             project.core.ui.echo("Creating a pyproject.toml for PDM...", style="primary")
@@ -89,15 +90,13 @@ class Command(BaseCommand):
                 "For more info, please visit https://peps.python.org/pep-0582/",
                 style="success",
             )
-        is_library = (
-            termui.confirm(
+        is_library = options.lib
+        if not is_library and self.interactive:
+            is_library = termui.confirm(
                 "Is the project a library that is installable?\n"
-                "A few more questions will be asked to include a project name "
-                "and build backend"
+                "If yes, we will need to ask a few more questions to include "
+                "the project name and build backend"
             )
-            if self.interactive
-            else False
-        )
         build_backend: type[BuildBackend] | None = None
         if is_library:
             name = self.ask("Project name", project.root.name)
@@ -115,11 +114,11 @@ class Command(BaseCommand):
                     prompt_type=int,
                     choices=[str(i) for i in range(len(all_backends))],
                     show_choices=False,
-                    default=all_backends.index("pdm-pep517"),
+                    default=0,
                 )
                 build_backend = get_backend(all_backends[int(selected_backend)])
             else:
-                build_backend = get_backend("pdm-pep517")
+                build_backend = DEFAULT_BACKEND
         else:
             name, version, description = "", "", ""
         license = self.ask("License(SPDX name)", "MIT")
