@@ -3,6 +3,7 @@ import argparse
 from packaging.version import Version
 
 from pdm.cli.commands.base import BaseCommand
+from pdm.cli.options import venv_option
 from pdm.exceptions import PdmUsageError
 from pdm.models.candidates import Candidate
 from pdm.models.project_info import ProjectInfo
@@ -22,6 +23,7 @@ class Command(BaseCommand):
     metadata_keys = ["name", "version", "summary", "license", "platform", "keywords"]
 
     def add_arguments(self, parser: argparse.ArgumentParser) -> None:
+        venv_option.add_to_parser(parser)
         parser.add_argument(
             "package",
             type=normalize_name,
@@ -48,13 +50,13 @@ class Command(BaseCommand):
                 return
             latest_stable = next(filter(filter_stable, matches), None)
             metadata = latest.prepare(project.environment).metadata
-            project_info = ProjectInfo.from_distribution(metadata)
         else:
             if not project.name:
                 raise PdmUsageError("This project is not a package")
             package = normalize_name(project.name)
-            project_info = ProjectInfo.from_metadata(project.pyproject.metadata)
+            metadata = project.make_self_candidate(False).prepare(project.environment).prepare_metadata(True)
             latest_stable = None
+        project_info = ProjectInfo.from_distribution(metadata)
 
         if any(getattr(options, key, None) for key in self.metadata_keys):
             for key in self.metadata_keys:
