@@ -43,7 +43,7 @@ from pdm.utils import (
 if TYPE_CHECKING:
     from unearth import Package, PackageFinder
 
-    from pdm.models.environment import Environment
+    from pdm.environments import BaseEnvironment
 
 
 def _dist_info_files(whl_zip: ZipFile) -> list[str]:
@@ -144,7 +144,7 @@ class Candidate:
         self.name = name or self.req.project_name
         self.version = version
         if link is None and not req.is_named:
-            link = req.as_file_link()  # type: ignore[attr-defined]
+            link = cast(Link, req.as_file_link())  # type: ignore[attr-defined]
         self.link = link
         self.summary = ""
         self.hashes: dict[Link, str] | None = None
@@ -257,7 +257,7 @@ class Candidate:
         """Format for output."""
         return f"[req]{self.name}[/] [warning]{self.version}[/]"
 
-    def prepare(self, environment: Environment) -> PreparedCandidate:
+    def prepare(self, environment: BaseEnvironment) -> PreparedCandidate:
         """Prepare the candidate for installation."""
         if self._prepared is None:
             self._prepared = PreparedCandidate(self, environment)
@@ -269,7 +269,7 @@ class PreparedCandidate:
     The metadata and built wheel are available.
     """
 
-    def __init__(self, candidate: Candidate, environment: Environment) -> None:
+    def __init__(self, candidate: Candidate, environment: BaseEnvironment) -> None:
         self.candidate = candidate
         self.environment = environment
         self.req = candidate.req
@@ -423,7 +423,7 @@ class PreparedCandidate:
                     self._source_dir = Path(build_dir)
                     self._unpacked_dir = result
 
-    def prepare_metadata(self) -> im.Distribution:
+    def prepare_metadata(self, force_build: bool = False) -> im.Distribution:
         self.obtain(allow_all=True)
         metadir_parent = create_tracked_tempdir(prefix="pdm-meta-")
         if self.wheel:
@@ -434,7 +434,7 @@ class PreparedCandidate:
         assert self._unpacked_dir, "Source directory isn't ready yet"
         # Try getting from PEP 621 metadata
         pyproject_toml = self._unpacked_dir / "pyproject.toml"
-        if pyproject_toml.exists():
+        if pyproject_toml.exists() and not force_build:
             pyproject = PyProject(pyproject_toml, ui=self.environment.project.core.ui)
             metadata = pyproject.metadata.unwrap()
             if not metadata:
@@ -486,7 +486,7 @@ class PreparedCandidate:
             except Exception:
                 message = "Failed to parse the project files, dependencies may be missing"
                 termui.logger.warn(message)
-                warnings.warn(message, RuntimeWarning)
+                warnings.warn(message, RuntimeWarning, stacklevel=1)
                 setup = Setup()
             return setup.as_dist()
         else:
@@ -548,14 +548,14 @@ class PreparedCandidate:
         if self.req.editable:
             # In this branch the requirement must be an editable VCS requirement.
             # The repository will be unpacked into a *persistent* src directory.
-            if self.environment.packages_path:
-                src_dir = self.environment.packages_path / "src"
+            if self.environment.is_local:
+                prefix = self.environment.packages_path  # type: ignore[attr-defined]
             else:
-                venv_prefix = get_venv_like_prefix(self.environment.interpreter.executable)
-                if venv_prefix is not None:
-                    src_dir = venv_prefix / "src"
-                else:
-                    src_dir = Path("src")
+                prefix = get_venv_like_prefix(self.environment.interpreter.executable)
+            if prefix is not None:
+                src_dir = prefix / "src"
+            else:
+                src_dir = Path("src")
             src_dir.mkdir(exist_ok=True, parents=True)
             dirname = self.candidate.name or self.req.name
             if not dirname:
